unit RadixConv;

interface

uses
  SysUtils, Math;

type
  TRadixRange  = 2..256;
  TDigitsRange = 0..255;

const
  FRadixTable = '0123456789ABCDEF';

// l -> n i
function IntToStrEx(const Value: Int64; const Radix: TRadixRange = 10; const Digits: TDigitsRange = 0; const RadixTable: string = ''): string;
// n i -> l
function StrToIntEx(const s: string; const Radix: TRadixRange = 10; const RadixTable: string = ''): Int64;

// l -> Excel 
function IntToExcelHexacos(const Value: Int64): string;
// Excel  -> l
function ExcelHexacosToInt(const s: string): Int64;

implementation

function IntToStrEx(const Value: Int64; const Radix: TRadixRange = 10; const Digits: TDigitsRange = 0; const RadixTable: string = ''): string;
// -----------------------------------------------------------------------------
// l -> n i
//
// 
//   Value     : ϊ鐔l
//   Radix     :  (Radix i) Radix >= 2
//   Digits    : \ (0 Ŏ)
//   RadixTable: e[u
//
// ߂l
//   Radix i
// -----------------------------------------------------------------------------
var
  i: Integer;
  dRadixTable: String;
  dValue: UInt64;
begin
  result := '';
  // Radix Table
  if RadixTable = '' then
    dRadixTable := FRadixTable
  else
    dRadixTable := RadixTable;
  // Error
  if Length(dRadixTable) < Radix then
    Exit;
  // Calc
  dValue := Value;
  for i:=1 to 64 do
    begin
      result := dRadixTable[(dValue mod Radix) + 1] + result;
      dValue := dValue div Radix;
      if dValue = 0 then
        Break;
      if (Digits > 0) and (i = Digits) then
        Break;
    end;
  // Format
  if Digits > 0 then
    begin
      result := Copy(result, Min(1, Digits - Length(result) + 1), Digits);
      result := StringOfChar(dRadixTable[1], Digits - Length(result)) + result;
    end;
end;

function StrToIntEx(const s: string; const Radix: TRadixRange = 10; const RadixTable: string = ''): Int64;
// -----------------------------------------------------------------------------
// n i -> l
//
// 
//   s         : Radix i
//   Radix     :  (Radix i) Radix >= 2
//   RadixTable: e[u
//
// ߂l
//   l
// -----------------------------------------------------------------------------
var
  i: Byte;
  dRadixTable: String;
  dValue: Byte;
  Loop: Byte;
  s2: string;
begin
  result := 0;
  // Radix Table
  if RadixTable = '' then
    dRadixTable := FRadixTable
  else
    dRadixTable := RadixTable;
  // Error
  if Length(dRadixTable) < Radix then
    Exit;
  // Loop Counter
  s2 := Trim(s);
  for i:=1 to Length(s2) do
    if s2[i] = dRadixTable[1] then
      s2[i] := ' '
    else
      Break;
  s2 := Trim(s2);
  Loop := Length(s2);
  // Calc
  for i:=1 to Loop do
    begin
      result := result * Radix;
      dValue := Pos(s2[i], dRadixTable);
      if (dValue = 0) or (dValue > Radix) then
        begin
          result := 0;
          Break;
        end;
      result := result + (dValue - 1);
    end;
end;

function IntToExcelHexacos(const Value: Int64): string;
// -----------------------------------------------------------------------------
// l -> Excel 
//
// 
//   Value : ϊ鐔l
//
// ߂l
//   Excel ` 26 i
// -----------------------------------------------------------------------------
const
  RadixTable = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
var
  dValue: UInt64;
  dDigits: TDigitsRange;
begin
  // Calc
  dValue  := 0;
  dDigits := 0;
  repeat
    Inc(dDigits);
    dValue := dValue + Trunc(IntPower(26, dDigits));
  until (Value < dValue);
  dValue := dValue - Trunc(IntPower(26, dDigits));
  result := IntToStrEx(Value - dValue, 26, dDigits, RadixTable);
end;

function ExcelHexacosToInt(const s: string): Int64;
// -----------------------------------------------------------------------------
// Excel  -> l
//
// 
//   s : Excel ` 26 i
//
// ߂l
//   l
// -----------------------------------------------------------------------------
const
  RadixTable = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
var
  i: Integer;
  s2: string;
begin
  s2 := Trim(s);
  // Calc
  result := StrToIntEx(s2, 26, RadixTable);
  for i:=1 to Length(s2)-1 do
    result := result + Trunc(IntPower(26, i));
end;
end.
